"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _slonik = require("slonik");

var _lodash = require("lodash");

var _Logger = _interopRequireDefault(require("../Logger"));

var _errors = require("../errors");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const log = _Logger.default.child({
  namespace: 'getByIds'
});

var _default = async (connection, tableName, ids, idName = 'id', identifiers, resultIsArray) => {
  let rows = [];

  if (ids.length > 0) {
    const idType = typeof ids[0] === 'number' ? 'int4' : 'text';
    rows = await connection.any(_slonik.sql`
      SELECT ${_slonik.sql.raw(identifiers)}
      FROM ${_slonik.sql.identifier([tableName])}
      WHERE ${_slonik.sql.identifier([idName])} = ANY(${_slonik.sql.array(ids, idType)})
    `);
  }

  const results = [];
  const targetPropertyName = (0, _lodash.camelCase)(idName);

  if (resultIsArray) {
    for (const id of ids) {
      let _result = [];

      for (let _key = 0, _length = rows.length, _value; _key < _length; ++_key) {
        _value = rows[_key];
        if (_value[targetPropertyName] === id) _result.push(_value);
      }

      const result = _result;
      results.push(result);
    }
  } else {
    for (const id of ids) {
      let _result2;

      for (let _key2 = 0, _length2 = rows.length, _value2; _key2 < _length2; ++_key2) {
        _value2 = rows[_key2];

        if (_value2[targetPropertyName] === id) {
          _result2 = _value2;
          break;
        }
      }

      let result = _result2;

      if (!result) {
        log.warn({
          id,
          idName,
          tableName
        }, 'resource not found');
        result = new _errors.NotFoundError();
      }

      results.push(result);
    }
  }

  return results;
};

exports.default = _default;
//# sourceMappingURL=getByIds.js.map