"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _pluralize = _interopRequireDefault(require("pluralize"));

var _lodash = require("lodash");

var _errors = require("../errors");

var _Logger = _interopRequireDefault(require("../Logger"));

var _generateFlowTypeDocument = _interopRequireDefault(require("./generateFlowTypeDocument"));

var _indent = _interopRequireDefault(require("./indent"));

var _isJoiningTable = _interopRequireDefault(require("./isJoiningTable"));

var _createColumnSelector = _interopRequireDefault(require("./createColumnSelector"));

var _createLoaderTypePropertyDeclaration = _interopRequireDefault(require("./createLoaderTypePropertyDeclaration"));

var _pluralizeTableName = _interopRequireDefault(require("./pluralizeTableName"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const log = _Logger.default.child({
  namespace: 'generateDataLoaderFactory'
});

const createLoaderByIdsDeclaration = (loaderName, tableName, keyColumnName, columnSelector, resultIsArray) => {
  return `const ${loaderName} = new DataLoader((ids) => {
  return getByIds(connection, '${tableName}', ids, '${keyColumnName}', '${columnSelector}', ${String(resultIsArray)});
}, dataLoaderConfigurationMap.${loaderName});`;
};

const createLoaderByIdsUsingJoiningTableDeclaration = (loaderName, joiningTableName, targetResourceTableName, joiningKeyName, lookupKeyName, columnSelector) => {
  return `const ${loaderName} = new DataLoader((ids) => {
  return getByIdsUsingJoiningTable(connection, '${joiningTableName}', '${targetResourceTableName}', '${joiningKeyName}', '${lookupKeyName}', '${columnSelector}', ids);
}, dataLoaderConfigurationMap.${loaderName});`;
}; // eslint-disable-next-line complexity


var _default = (unnormalisedColumns, indexes, dataTypeMap) => {
  const columns = unnormalisedColumns.map(column => {
    return _objectSpread({}, column, {
      mappedTableName: column.mappedTableName || column.tableName
    });
  });

  if (columns.length === 0) {
    throw new _errors.UnexpectedStateError('Must know multiple columns.');
  }

  const tableNames = columns.map(column => {
    return column.tableName;
  }).filter((tableName, index, self) => {
    return self.indexOf(tableName) === index;
  });
  const loaders = [];
  const loaderNames = [];
  const loaderTypes = [];

  for (const tableName of tableNames) {
    const tableColumns = columns.filter(column => {
      return column.tableName === tableName;
    });

    if (tableColumns.length === 0) {
      continue;
    }

    const mappedTableName = tableColumns[0].mappedTableName;
    const resouceName = (0, _lodash.upperFirst)((0, _lodash.camelCase)(mappedTableName));

    for (const tableColumn of tableColumns) {
      const tableColumnSelector = (0, _createColumnSelector.default)(tableColumns);

      if (tableColumn.name.endsWith('_id')) {
        const loaderName = (0, _pluralize.default)(resouceName) + 'By' + (0, _lodash.upperFirst)((0, _lodash.camelCase)(tableColumn.name)) + 'Loader';
        loaders.push(createLoaderByIdsDeclaration(loaderName, tableName, tableColumn.name, tableColumnSelector, true));
        const loaderType = (0, _createLoaderTypePropertyDeclaration.default)(loaderName, dataTypeMap[tableColumn.dataType] ? dataTypeMap[tableColumn.dataType] : tableColumn.dataType, tableColumn.mappedTableName, true);
        loaderTypes.push(loaderType);
        loaderNames.push(loaderName);
      }
    }

    const tableUniqueIndexes = indexes.filter(index => {
      return index.tableName === tableName && index.indexIsUnique === true && index.columnNames.length === 1;
    });

    for (const tableUniqueIndex of tableUniqueIndexes) {
      const tableColumnSelector = (0, _createColumnSelector.default)(tableColumns);
      const maybeIndexColumnName = tableUniqueIndex.columnNames[0];

      if (!maybeIndexColumnName) {
        throw new Error('Unexpected state.');
      }

      const indexColumn = tableColumns.find(column => {
        return column.name === maybeIndexColumnName;
      });

      if (!indexColumn) {
        throw new Error('Unexpected state.');
      }

      const loaderName = resouceName + 'By' + (0, _lodash.upperFirst)((0, _lodash.camelCase)(indexColumn.name)) + 'Loader';
      loaders.push(createLoaderByIdsDeclaration(loaderName, tableName, indexColumn.name, tableColumnSelector, false));
      const loaderType = (0, _createLoaderTypePropertyDeclaration.default)(loaderName, dataTypeMap[indexColumn.dataType] ? dataTypeMap[indexColumn.dataType] : indexColumn.dataType, indexColumn.mappedTableName, false);
      loaderTypes.push(loaderType);
      loaderNames.push(loaderName);
    }
  }

  for (const tableName of tableNames) {
    const tableColumns = columns.filter(column => {
      return column.tableName === tableName;
    });

    if (tableColumns.length === 0) {
      continue;
    }

    if (!(0, _isJoiningTable.default)(tableName, tableColumns)) {
      continue;
    }

    const firstIdColumnNames = tableColumns.map(column => {
      return column.name;
    }).filter(columnName => {
      return columnName.endsWith('_id');
    }).map(columnName => {
      return columnName.slice(0, -3);
    }).slice(0, 2);

    if (firstIdColumnNames.length < 2) {
      throw new Error('Unexpected state.');
    }

    const relations = [{
      key: firstIdColumnNames[0],
      resource: firstIdColumnNames[1]
    }, {
      key: firstIdColumnNames[1],
      resource: firstIdColumnNames[0]
    }];

    for (const relation of relations) {
      const loaderName = (0, _lodash.upperFirst)((0, _lodash.camelCase)((0, _pluralizeTableName.default)(relation.resource))) + 'By' + (0, _lodash.upperFirst)((0, _lodash.camelCase)(relation.key + '_id')) + 'Loader';

      if (loaderNames.includes(loaderName)) {
        continue;
      }

      const resourceTableColumns = columns.filter(column => {
        return column.mappedTableName === relation.resource;
      });

      if (!resourceTableColumns.length) {
        log.warn({
          relation
        }, 'resource without columns');
        continue;
      }

      const realResourceTableName = resourceTableColumns[0].tableName;
      const tableColumnSelector = (0, _createColumnSelector.default)(resourceTableColumns, 'r2');
      loaders.push(createLoaderByIdsUsingJoiningTableDeclaration(loaderName, tableName, realResourceTableName, relation.resource, relation.key, tableColumnSelector));
      const keyColumn = tableColumns.find(column => {
        return column.name === relation.key + '_id';
      });

      if (!keyColumn) {
        throw new Error('Unexpected state.');
      }

      const loaderType = (0, _createLoaderTypePropertyDeclaration.default)(loaderName, dataTypeMap[keyColumn.dataType] ? dataTypeMap[keyColumn.dataType] : keyColumn.dataType, relation.resource, true);
      loaderTypes.push(loaderType);
      loaderNames.push(loaderName);
    }
  }

  loaderTypes.sort((a, b) => {
    return a.localeCompare(b);
  });
  loaderNames.sort((a, b) => {
    return a.localeCompare(b);
  });
  return `// @flow

import {
  getByIds,
  getByIdsUsingJoiningTable
} from 'postloader';
import DataLoader from 'dataloader';
import type {
  DatabaseConnectionType
} from 'slonik';
${(0, _generateFlowTypeDocument.default)(columns, dataTypeMap)}

export type LoadersType = {|
${loaderTypes.map(body => {
    return (0, _indent.default)(body, 2);
  }).sort().join(',\n')}
|};

export const createLoaders = (connection: DatabaseConnectionType, dataLoaderConfigurationMap: Object = {}): LoadersType => {
${loaders.map(body => {
    return (0, _indent.default)(body, 2);
  }).join('\n')}

  return {
${loaderNames.map(body => {
    return (0, _indent.default)(body, 4);
  }).join(',\n')}
  };
};`;
};

exports.default = _default;
//# sourceMappingURL=generateDataLoaderFactory.js.map